---DECLARE @LastGroupMessagesGlobalEventId AS BIGINT
---#DECLARE @LastGroupEndpointRosterGlobalEventId AS BIGINT
---DECLARE @LastGroupUserRightsGlobalEventId AS BIGINT

---SET @LastGroupMessagesGlobalEventId = 0;
---SET @LastGroupEndpointRosterGlobalEventId = 0;
---SET @LastGroupUserRightsGlobalEventId = 0;

-- Generates a temporary table containing pairs of Group IDs and the JSON Payload of it's Security Contexts.
SELECT
	esc.Id,
	TRIM('[]' FROM asce.SecurityContextPayload) AS SecurityContextPayload INTO #TempGroupToSingularSecurityContextPayload
FROM
	(
		SELECT * 
		FROM (
			SELECT
				Id,
				Trim('[]"' FROM value) AS SecurityContextId
			FROM GroupProperties CROSS APPLY STRING_SPLIT(SecurityContexts, ',')
		) esc
		WHERE esc.SecurityContextId NOT LIKE '(null)') esc,
    (
		SELECT Id as SecurityContextId,
		(
			SELECT
				Id,
				DisplayName,
				AttributeProvider,
				AttributeName,
				AttributeValue
			FROM AdminSecurityContextEntries
			WHERE asce.Id = AdminSecurityContextEntries.Id FOR JSON PATH) AS SecurityContextPayload
		FROM AdminSecurityContextEntries asce
	) asce
WHERE esc.SecurityContextId = asce.SecurityContextId;

WITH CTE_SecurityContextTable AS (
	SELECT
		Id,
		SecurityContextPayload
    FROM #TempGroupToSingularSecurityContextPayload)
SELECT
	table0.Id,
	CONCAT('[', STUFF(
		(
			SELECT ',' + table1.SecurityContextPayload
			FROM CTE_SecurityContextTable table1
			WHERE table1.Id = table0.Id
			ORDER BY table1.SecurityContextPayload
			FOR XML PATH('')), 1, LEN(','), ''), ']') AS SecurityContexts INTO #TempGroupToPayload
FROM CTE_SecurityContextTable table0
GROUP BY table0.Id
ORDER BY Id;

SELECT
	Users.Id as UserId,
	Users.DisplayName as UserDisplayName,
	Users.LinkedUserIdentity,
	Endpoints.Id as EndpointId
INTO #TempUserEndpointMap
FROM Users
INNER JOIN Endpoints
ON Endpoints.UserId = Users.Id

-- Initial query for extracting all required data
SELECT 
	EventName,
	Timestamp,
	Timestamp as Timestring,
	ModelId AS GroupId,
	EventId AS MessageId,
	GroupName,
	SecurityContexts AS GroupSecurityContexts,
	UserId,
	UserDisplayName,
	LinkedUserIdentity
FROM
	(
		SELECT
			MessagesEvents.EventName,
			MessagesEvents.Timestamp,
			MessagesEvents.ModelId,
			MessagesEvents.EventId,
			GroupProperties.Name AS GroupName,
			#TempGroupToPayload.SecurityContexts,
			Users.Id AS UserId,
			Users.DisplayName AS UserDisplayName,
			Users.LinkedUserIdentity
		FROM MessagesEvents INNER JOIN
			GroupProperties ON MessagesEvents.ModelId = GroupProperties.Id INNER JOIN
			Users ON MessagesEvents.SenderId = Users.Id INNER JOIN
			#TempGroupToPayload ON GroupProperties.Id = #TempGroupToPayload.Id
		WHERE
			MessagesEvents.EventName = 'MessageReceivedEventData'
			AND
			MessagesEvents.GlobalEventId > @LastGroupMessagesGlobalEventId

	UNION
		SELECT
			GroupEndpointRosterEvents.EventName,
			GroupEndpointRosterEvents.Timestamp,
			GroupEndpointRosterEvents.ModelId,
			null AS EventId,
			GroupProperties.Name AS GroupName,
			#TempGroupToPayload.SecurityContexts,
			#TempUserEndpointMap.UserId,
			#TempUserEndpointMap.UserDisplayName,
			#TempUserEndpointMap.LinkedUserIdentity
		FROM GroupEndpointRosterEvents 
		INNER JOIN #TempGroupToPayload ON GroupEndpointRosterEvents.ModelId = #TempGroupToPayload.Id 
		INNER JOIN #TempUserEndpointMap ON JSON_VALUE(GroupEndpointRosterEvents.EventPayload, '$.EndpointId') = #TempUserEndpointMap.EndpointId
		INNER JOIN GroupProperties ON GroupProperties.Id = GroupEndpointRosterEvents.EntityId
		WHERE
			(GroupEndpointRosterEvents.EventName = 'EndpointJoinedEventData' OR GroupEndpointRosterEvents.EventName = 'EndpointLeftEventData')
			AND
			GroupEndpointRosterEvents.GlobalEventId > @LastGroupEndpointRosterGlobalEventId

	UNION
		SELECT
			GroupUserRightsEvents.EventName,
			GroupUserRightsEvents.Timestamp,
			GroupUserRightsEvents.ModelId,
			null AS EventId,
			GroupProperties.Name AS GroupName, 
			#TempGroupToPayload.SecurityContexts,
			Users.Id AS UserId,
			Users.DisplayName AS UserDisplayName,
			Users.LinkedUserIdentity
		FROM
			GroupUserRightsEvents INNER JOIN
				GroupProperties ON GroupUserRightsEvents.ModelId = GroupProperties.Id INNER JOIN
				Users ON JSON_VALUE(GroupUserRightsEvents.EventPayload, '$.UserId') = Users.Id INNER JOIN
				#TempGroupToPayload ON GroupProperties.Id = #TempGroupToPayload.Id
		WHERE
			(GroupUserRightsEvents.EventName = 'ReadRightAddedEventData' OR GroupUserRightsEvents.EventName = 'ReadRightRemovedEventData')
			AND
			GroupUserRightsEvents.GlobalEventId > @LastGroupUserRightsGlobalEventId
	) as AuditEvents 
ORDER BY AuditEvents.Timestamp

SELECT
	'MessagesEvents' AS TableName,
	MAX(MessagesEvents.GlobalEventId) AS LatestGlobalEventId
FROM 
	MessagesEvents

UNION

SELECT
	'GroupEndpointRosterEvents' AS TableName,
	MAX(GroupEndpointRosterEvents.GlobalEventId) AS LatestGlobalEventId
FROM 
	GroupEndpointRosterEvents

UNION

SELECT
	'GroupUserRightsEvents' AS TableName,
	MAX(GroupUserRightsEvents.GlobalEventId) AS LatestGlobalEventId
FROM 
	GroupUserRightsEvents

-- Clean up temporary tables
IF OBJECT_ID(N'tempdb..#TempGroupToPayload') IS NOT NULL
	BEGIN
		DROP TABLE #TempGroupToPayload
	END

IF OBJECT_ID(N'tempdb..#TempGroupToSingularSecurityContextPayload') IS NOT NULL
	BEGIN
		DROP TABLE #TempGroupToSingularSecurityContextPayload
	END

IF OBJECT_ID(N'tempdb..#TempUserEndpointMap') IS NOT NULL
	BEGIN
		DROP TABLE #TempUserEndpointMap
	END
