<#
.Description
Queries the MCE database and outputs relavent audit events to a JSON file for when a user:
- Gains read rights to a group
- Loses read rights to a group
- Joins a group
- Leaves a group
- Sends a message

Upon running the script a "LastEvent.txt" file will be generated in the working directory of the script.
This file contains cursor information about the previous execution, which is used in subsequent executions to retrieve the new events since the last execution.

.PARAMETER DatabaseConnectionString
Mandatory parameter used to determine the connection string to connect to the MCE database. 

.PARAMETER SqlScriptPath
Mandatory parameter used to specify the path to the SQL script.
The script should have been included packaged with this PowerShell script.

.PARAMETER Timeout
Optional parameter used to specify the wait time (in seconds) before terminating the attempt to execute the sql script and generating an error.
If not specified then the parameter will default 0, indicating no timeout limit.

.PARAMETER OutFile
Optional parameter which specifies the output file which contains the generated audit events.
If not specified, will default to "AuditResults{CurrentDateTimestamp}.json" saving a file within the running directory, where the timestamps are derived from the time the script is run.

.EXAMPLE
PS> .\InvokeGenerateAuditEvents.ps1 -DatabaseConnectionString 'Server=.;Initial Catalog=MCE;Integrated Security=SSPI' -SqlScriptPath 'C:\eng\GenerateAuditEvents.sql'
.EXAMPLE
PS> .\InvokeGenerateAuditEvents.ps1 -DatabaseConnectionString 'Server=;Initial Catalog=mce-test;User Id=myUsername;Password=myPassword;' -SqlScriptPath 'GenerateAuditEvents.sql' -Timeout 300

.SYNOPSIS
Generates relavent audit events from the given database.
#>

param (
    [string] [Parameter(Mandatory)] $DatabaseConnectionString,
    [string] [Parameter(Mandatory)] $SqlScriptPath,
    [int] $Timeout=0,
    [string] $OutFile
);
Import-Module ./GenerateAuditEvents.ps1 -Force
Get-AuditEvents -DatabaseConnectionString $DatabaseConnectionString -SqlScriptPath $SqlScriptPath -Timeout $Timeout | Save-AuditEvents -OutFile $OutFile