___Included should be a PowerShell and 2 SQL scripts.___

---


### Description
Included in the PowerShell module are three commandlets:
- Update-DatabaseForAuditing
   - Updates event tables to include a unique ID column to the event tables to be audited.
- Get-AuditEvents
   - Produces a list of objects, each representing an audit event from the database.
- Save-AuditEvents
   - Takes in a list of objects representing the audit events to save into a particular file.


Running all functions correctly will produce a singular _json_ file containing audit records for when a user:
- Gains read rights to a group
- Loses read rights to a group
- Joins a group
- Leaves a group
- Sends a message

If not specified, this filename of the results will contain the unix timestamp of the time at which running all functions was executed.
If attempting to save results with a duplicate filename, numbers will be appended to ensure uniqueness and that no file is overridden.

The records will be in chronological order specified by the timestamp of when the event occurred.
- Each record will contain the following information:
   - The type of the event
   - The time of the event
      - A timestamp as UNIX time
      - A human readable UTC timestring
   - The ID of the group
   - The name of the group
   - The security context of the group
   - The ID of the user
   - The name of the user
   - The linked identity of the user

If the event concerns a sent message, the message content will __not__ be included, instead the message ID will be included for further auditing purposes.

### How to run
It is not a requirement that the SQL script and PowerShell script be kept within the same working directory as full file paths can be specified for both the SQL script and output json file.

Before being able to access the inner functions, __it is required that you import__ the _GenerateAuditEvents.ps1_ script with:
<code>Import_Module ./GenerateAuditEvents.ps1</code>

When using this script for the first time in order to audit any events, _Update-DatabaseForAuditing_ __must be used to update the database.__
If not executed, you may come across an error messaging mentioning "Invalid column name 'GlobalEventId'". This column is required to fetch all the events from the relevent tables.

After successfully running the _Get-AuditEvents_ cmdlet, a "LastEvents.json" file will be generated in the working directory of the script.
This file contains cursor information about the previous execution, which is used in subsequent executions. This will retrieve the new events that have occurred since the last execution.
Deleting this file will cause the next run to retrieve all events.

Updating the database with __Update-DatabaseForAuditing__ specifies the following parameters:

- __DatabaseConnectionString:__
Mandatory parameter used to determine the connection string to connect to the MCE database.

- __SqlScriptPath:__
Mandatory parameter used to specify the path to the SQL script.
The script, "PrimeDbForAuditing.sql", should have been included packaged with this PowerShell script.

- __UpdateType:__
Mandatory parameter used to specify how you are updating the table columns. 
Values can be:
 - Add: case-insensitive, will add a unique id column in the tables of the events.
 - Remove: case-insensitive, will remove a unique id column in the tables of the events.

___Steps to prime the database for auditing:___
- Open PowerShell instance and navigate to script directory
- <code>Import-Module ./GenerateAuditEvents.ps1</code>
- Run _Update-DatabaseForAuditing_
<code>PS> Update-DatabaseForAuditing -DatabaseConnectionString 'Server=.;Initial Catalog=Mce;Integrated Security=SSPI' -SqlScriptPath "PrimeDbForAuditing.sql" -UpdateType "Add"</code>

___Steps to reset the database from auditing changes:___
- Open PowerShell instance and navigate to script directory
- <code>Import-Module ./GenerateAuditEvents.ps1</code>
- Run _Update-DatabaseForAuditing_
<code>PS> Update-DatabaseForAuditing -DatabaseConnectionString 'Server=.;Initial Catalog=Mce;Integrated Security=SSPI' -SqlScriptPath "PrimeDbForAuditing.sql" -UpdateType "Remove"</code>

Generating audit events with __Get-AuditEvents__ specifies the following parameters:

- __DatabaseConnectionString:__
Mandatory parameter used to determine the connection string to connect to the MCE database. 

- __SqlScriptPath:__
Mandatory parameter used to specify the path to the SQL script.
The script, "GenerateAuditEvents.sql", should have been included packaged with this PowerShell script.

- __Timeout:__
Optional parameter used to specify the wait time (in seconds) before terminating the attempt to execute the sql script and generating an error.
If not specified then the parameter will default 0, indicating no timeout limit.

___Steps:___
- Open PowerShell instance and navigate to script directory
- <code>Import-Module ./GenerateAuditEvents.ps1</code>
- Run _Get-AuditEvents_ using the parameters specified above (use Get-Help for more information on the commandlet)
<code>Get-AuditEvents -DatabaseConnectionString {MyDatabaseConnectionString} -SqlScriptPath {GenerateAuditEvents.sql}</code>


Saving audit events with __Save-AuditEvents__ specifies the following parameters:

- __AuditEvents:__
Mandatory parameter supplied by the pipeline, this is an ordered list of System.Object each representing an audit event entry.

- __OutFile:__
Optional parameter which specifies the output file which contains the generated audit events.
If not specified, will default to "AuditResults{CurrentTimestamp}.json" saving a file within the running directory, where the timestamp is derived from the time that the script runs.

___Steps:___
- Open PowerShell instance and navigate to script directory
- <code>Import-Module ./GenerateAuditEvents.ps1</code>
- Run _Save-AuditEvents_ using the output from _Get-AuditEvents_
<code>Get-AuditEvents -DatabaseConnectionString {MyDatabaseConnectionString} -SqlScriptPath {GenerateAuditEvents.sql} | Save-AuditEvents -OutFile {MyCustomFileName}</code>

---

### Examples
- _Running against a localhost for the first time, specifying the full path of the SQL script and saving the json file to the working directory_
<code>PS> Update-DatabaseForAuditing -DatabaseConnectionString 'Server=.;Initial Catalog=Mce;Integrated Security=SSPI' -SqlScriptPath "PrimeDbForAuditing.sql" -UpdateType "Add"
PS> Get-AuditEvents -DatabaseConnectionString 'Server=localhost;Initial Catalog=MCE;Integrated Security=SSPI' -SqlScriptPath 'GenerateAuditEvents.sql' | Save-AuditEvents</code>

- _Running against a DB with credentials, specifying the name of the SQL script within the same working directory and saving the json file to a separate directory_
<code> PS> .\GenerateAuditEvents.ps1 -DatabaseConnectionString 'Server=;Initial Catalog=mce-test;User Id=myUsername;Password=myPassword;' -SqlScriptPath 'C:\eng\GenerateAuditEvents.sql' | Save-AuditEvents -OutFile 'C:\files\MyCustomFileName'</code>
